/**
 * @file
 * $Id: daemon.h,v 1.11 2003/03/25 08:26:38 tsavola Exp $
 * @author Timo Savola <timo.savola@movial.fi>
 */

#ifndef DAEMON_H
#define DAEMON_H

#include <stdio.h>
#include <pwd.h>
#include <netdb.h>
#include "protocol.h"
#include "buffer.h"

/** Name of the config file in the user's home directory. */
#define CONFIG_NAME        ".sbrshd"

/** Gets printed with the --version option. */
#define DAEMON_VERSION     "Scratchbox Remote Shell daemon " VERSION

/** Backlog parameter for the listen() call. */
#define BACKLOG            16

/** How many seconds to wait between checking for expired mounts? */
#define TIMEOUT            (15 * 60)

/** How many seconds until a mount expires? */
#define EXPIRATION         (2 * 60 * 60)

/** The permissions used when creating directories for mount points. */
#define MKDIR_PERMS        0755

/** Default path to the mount binary. */
#define DEFAULT_MOUNT_CMD  "/bin/mount"

/** Default path to the umount binary. */
#define DEFAULT_UMOUNT_CMD "/bin/umount"

/** Default path to a file used for checking existing mounts. */
#define DEFAULT_MOUNT_TAB  "/etc/mtab"

/** Default option used with MTYPE_BIND. */
#define DEFAULT_BIND_OPT   "--bind"

/** NFS mount type for mount_t. */
#define MTYPE_NFS          1

/** Mount type for mount_t when a path should be bound to a mount point. */
#define MTYPE_BIND         2

struct mount_s;

/** An entry in a doubly linked list of mount points. */
typedef struct mount_s {
	struct mount_s *prev;
	struct mount_s *next;

	/** How many processes are using this mount? */
	int usage;

	/** When will the mount expire? Relevant when usage is 0. */
	time_t expiration;

	/** MTYPE_NFS or MTYPE_BIND. */
	int type;

	/** Mount point path. */
	char *point;

	/** Path to special file or whatever should be mounted... */
	char *device;

	/** Options for the mount command (-o). */
	char *opts;
} mount_t;

/** Handler process data. */
typedef struct {
	/** Socket connection to the client. */
	int sd;

	/** Host name of the client. */
	char host[NI_MAXHOST];

	/** Username and password from the client. */
	auth_info_t auth;

	/** User's uid, home dir, etc. from the system.
            tmp_buf is used to store the strings of this structure! */
	struct passwd passwd;

	/** Command description from the client. */
	cmd_info_t cmd;

	/** Root directory. */
	char *root;

	/** The mount points used by this client. */
	mount_t **mounts;

	/** Used for buffering stdin data received from the client. */
	buffer_t buf_in;

	/** Used for many things... */
	char tmp_buf[BUFFER_SIZE];

	/** Are we waiting for IN DATA? */
	int inwait;

	/** How much OUT DATA has the client requested? */
	int outreq;

	/** How much ERR DATA has the client requested? */
	int errreq;
} handler_t;

#endif
